#include "sokodan.h"
#include <string.h>
#include "MoveX.h"
#include "Move.h"


void Move_init (void)
{
  Move_Pool = g_mem_chunk_create (Move, MOVE_PREALLOC, G_ALLOC_AND_FREE);
}


MovePtr Move_new (void)
{
  return g_chunk_new (Move, Move_Pool);
}

void Move_delete (MovePtr move)
{
  g_chunk_free (move, Move_Pool);
}

static void _Move_delete (gpointer data, gpointer extra)
{
  Move_delete ((MovePtr) data);
  g_assert (extra == NULL);
}


/* Creates a list of moves */
MoveList MoveList_new (void)
{
	return NULL;
}

/*
  +----------------------------------------------------------------------------
  | NAME:     MoveList_delete
  | PURPOSE:  Frees all the moves in the lists, and frees the list itself.
  | INPUT:    A list of moves.
  | OUTPUT:   None.
  | NOTES:    The moves are *really* freed, the same way they have been
  |           allocated in .._add_move().
  +----------------------------------------------------------------------------
*/
void MoveList_delete (MoveList  move_list)
{
  g_slist_foreach (move_list, _Move_delete, NULL);
  g_slist_free (move_list);
}

/*
  +----------------------------------------------------------------------------
  | NAME:     MoveList_add_Move
  | PURPOSE:  Adds the move at the end of the list.
  | INPUT:    A pointer to a list of moves and a pointer to a move.
  | OUTPUT:   None.
  | NOTES:    The move is duplicated before being added to the list, thus
  |           the move can be a local variable.
  +----------------------------------------------------------------------------
*/
void MoveList_add_Move (MoveList     *move_list,
                        const MovePtr move)
{
	*move_list = g_slist_prepend (*move_list, _Move_duplicate (move));
}

/*
  +----------------------------------------------------------------------------
  | NAME:     _Move_duplicate
  | PURPOSE:  Duplicate a move.
  | INPUT:    A pointer to the move to duplicate.
  | OUTPUT:   A pointer to the new copy of the move.
  | NOTES:    None.
  +----------------------------------------------------------------------------
*/
static MovePtr _Move_duplicate (const MovePtr move)
{
  MovePtr duplicata = Move_new ();

  memcpy (duplicata, move, sizeof (Move));

  return duplicata;
}
