#include "sokodan.h"
#include "PosHashTableX.h"
#include "PosHashTable.h"
#include "PositionX.h"

/*
  +----------------------------------------------------------------------------
  | NAME:     PosHashTable_new
  | PURPOSE:  Creates a new hash table.
  | INPUT:    None.
  | OUTPUT:   A hash table.
  | NOTES:    
  +----------------------------------------------------------------------------
*/
PosHashTable PosHashTable_new (void)
{
	PosHashTable htable;

	htable = g_hash_table_new (gPosition_hash, gPosition_compare);
	g_hash_table_thaw (htable); /* Is it necessary (default behaviour?) ? */

	return htable;
}

/*
  +----------------------------------------------------------------------------
  | NAME:     PosHashTable_delete
  | PURPOSE:  Frees all the positions in the hash table and eventually destroys
  |           the hash table.
  | INPUT:    A hash table.
  | OUTPUT:   None.
  | NOTES:    The positions are allocated (duplicated) in .._add_Position.
  +----------------------------------------------------------------------------
*/
void PosHashTable_delete (PosHashTable pht)
{
	g_hash_table_foreach (pht, _PosHashTable_free_Position, NULL);
	g_hash_table_destroy (pht);
}

static void _PosHashTable_free_Position (gpointer key,
																				 gpointer value,
																				 gpointer data)
{
  if (value != NOT_NULL)
  {
    if (value != NULL)
      g_print ("Ouch, grosse corruption\n");
    else
      g_print ("Value  NULL\n");
  }
	/* g_assert (value == NOT_NULL); */
	g_assert (data  == NULL);

	Position_delete ((PositionPtr) key);
}

/*
  +----------------------------------------------------------------------------
  | NAME:     PosHashTable_add_Position
  | PURPOSE:  Checks if the position is already in the hash table, and inserts
  |           it if it wasn't.
  | INPUT:    A hash table and a pointer to a position.
  | OUTPUT:   A pointer to a duplicata of the input position if it has been
	|           inserted, NULL if it was already in the hash table.
  | NOTES:    The position is duplicated before being inserted.
  +----------------------------------------------------------------------------
*/
PositionPtr PosHashTable_add_Position (PosHashTable      pht,
                                       const PositionPtr pos)
{
  PositionPtr dup;

	if (g_hash_table_lookup (pht, pos) != NULL)
		return NULL;

  dup = Position_duplicate (pos);
	g_hash_table_insert (pht, dup, NOT_NULL);
	return dup;
}

/*
  +----------------------------------------------------------------------------
  | NAME:     PosHashTable_lookup_Position
  | PURPOSE:  Returns the position in the hash table that is equal to the
  |           position passed as a parameter.
  | INPUT:    A hash table and a position.
  | OUTPUT:   The same position, but the one in the hash table.
  | NOTES:    Not used anymore...
  +----------------------------------------------------------------------------
*/
PositionPtr PosHashTable_lookup_Position (const PosHashTable pht,
																					const PositionPtr pos)
{
	gboolean result;
	gpointer orig_key, value;

	result = g_hash_table_lookup_extended (pht, pos, &orig_key, &value);
	g_assert (result == TRUE);
	g_assert (value == NOT_NULL);

	return (PositionPtr) orig_key;
}
